@@ -9,11 +9,11 @@
 #include <errno.h>
 #include <limits.h>
 #include <signal.h>
+#include <poll.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/sysmacros.h>
-#include <sys/poll.h>
 #include <linux/major.h>
 #include <sys/time.h>
 typedef unsigned char u_char;   /* horrible, for scsi.h */
@@ -47,7 +47,8 @@ typedef unsigned char u_char;   /* horrible, for scsi.h */
 
 */
 
-static char * version_str = "0.56 20140409";    /* was "0.55 20020509" */
+static char * version_str = "0.57 20140819";
+/* resurrected from "0.55 20020509" */
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
@@ -144,15 +145,11 @@ static int dd_count = -1;
 
 static const char * proc_allow_dio = "/proc/scsi/sg/allow_dio";
 
-int sg_fin_in_operation(Rq_coll * clp, Rq_elem * rep);
-int sg_fin_out_operation(Rq_coll * clp, Rq_elem * rep);
-int normal_in_operation(Rq_coll * clp, Rq_elem * rep, int blocks);
-int normal_out_operation(Rq_coll * clp, Rq_elem * rep, int blocks);
-int sg_start_io(Rq_elem * rep);
-int sg_finish_io(int wr, Rq_elem * rep);
+static int sg_finish_io(int wr, Rq_elem * rep);
 
 
-static void install_handler (int sig_num, void (*sig_handler) (int sig))
+static void
+install_handler (int sig_num, void (*sig_handler) (int sig))
 {
     struct sigaction sigact;
     sigaction (sig_num, NULL, &sigact);
@@ -165,7 +162,8 @@ static void install_handler (int sig_num, void (*sig_handler) (int sig))
     }
 }
 
-void print_stats()
+static void
+print_stats()
 {
     int infull, outfull;
 
@@ -177,7 +175,8 @@ void print_stats()
     fprintf(stderr, "%d+%d records out\n", outfull, rcoll.out_partial);
 }
 
-static void interrupt_handler(int sig)
+static void
+interrupt_handler(int sig)
 {
     struct sigaction sigact;
 
@@ -190,14 +189,16 @@ static void interrupt_handler(int sig)
     kill (getpid (), sig);
 }
 
-static void siginfo_handler(int sig)
+static void
+siginfo_handler(int sig)
 {
     fprintf(stderr, "Progress report, continuing ...\n");
     print_stats ();
     if (sig) { }        /* suppress unused warning */
 }
 
-int dd_filetype(const char * filename)
+static int
+dd_filetype(const char * filename)
 {
     struct stat st;
 
@@ -212,7 +213,8 @@ int dd_filetype(const char * filename)
     return FT_OTHER;
 }
 
-void usage()
+static void
+usage()
 {
     fprintf(stderr, "Usage: "
            "sgq_dd  [if=<infile>] [skip=<n>] [of=<ofile>] [seek=<n>] "
@@ -232,7 +234,8 @@ void usage()
 }
 
 /* Returns -1 for error, 0 for nothing found, QS_IN_POLL or QS_OUT_POLL */
-int do_poll(Rq_coll * clp, int timeout, int * req_indexp)
+static int
+do_poll(Rq_coll * clp, int timeout, int * req_indexp)
 {
     int k, res;
 
@@ -277,7 +280,8 @@ int do_poll(Rq_coll * clp, int timeout, int * req_indexp)
 
 
 /* Return of 0 -> success, -1 -> failure, 2 -> try again */
-int read_capacity(int sg_fd, int * num_sect, int * sect_sz)
+static int
+read_capacity(int sg_fd, int * num_sect, int * sect_sz)
 {
     int res;
     unsigned char rcCmdBlk [10] = {0x25, 0, 0, 0, 0, 0, 0, 0, 0, 0};
@@ -319,7 +323,8 @@ int read_capacity(int sg_fd, int * num_sect, int * sect_sz)
 }
 
 /* 0 -> ok, 1 -> short read, -1 -> error */
-int normal_in_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
+static int
+normal_in_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
 {
     int res;
     int stop_after_write = 0;
@@ -357,7 +362,8 @@ int normal_in_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
 }
 
 /* 0 -> ok, -1 -> error */
-int normal_out_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
+static int
+normal_out_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
 {
     int res;
 
@@ -387,7 +393,8 @@ int normal_out_operation(Rq_coll * clp, Rq_elem * rep, int blocks)
 }
 
 /* Returns 1 for retryable, 0 for ok, -ve for error */
-int sg_fin_in_operation(Rq_coll * clp, Rq_elem * rep)
+static int
+sg_fin_in_operation(Rq_coll * clp, Rq_elem * rep)
 {
     int res;
 
@@ -416,7 +423,8 @@ int sg_fin_in_operation(Rq_coll * clp, Rq_elem * rep)
 }
 
 /* Returns 1 for retryable, 0 for ok, -ve for error */
-int sg_fin_out_operation(Rq_coll * clp, Rq_elem * rep)
+static int
+sg_fin_out_operation(Rq_coll * clp, Rq_elem * rep)
 {
     int res;
 
@@ -443,7 +451,8 @@ int sg_fin_out_operation(Rq_coll * clp, Rq_elem * rep)
     return res;
 }
 
-int sg_start_io(Rq_elem * rep)
+static int
+sg_start_io(Rq_elem * rep)
 {
     sg_io_hdr_t * hp = &rep->io_hdr;
     int res;
@@ -491,7 +500,8 @@ int sg_start_io(Rq_elem * rep)
 }
 
 /* -1 -> unrecoverable error, 0 -> successful, 1 -> try again */
-int sg_finish_io(int wr, Rq_elem * rep)
+static int
+sg_finish_io(int wr, Rq_elem * rep)
 {
     int res;
     sg_io_hdr_t io_hdr;
@@ -555,7 +565,8 @@ int sg_finish_io(int wr, Rq_elem * rep)
 }
 
 /* Returns scsi_type or -1 for error */
-int sg_prepare(int fd, int sz)
+static int
+sg_prepare(int fd, int sz)
 {
     int res, t;
     struct sg_scsi_id info;
@@ -584,7 +595,8 @@ int sg_prepare(int fd, int sz)
 }
 
 /* Return 0 for ok, anything else for errors */
-int prepare_rq_elems(Rq_coll * clp, const char * inf, const char * outf)
+static int
+prepare_rq_elems(Rq_coll * clp, const char * inf, const char * outf)
 {
     int k;
     Rq_elem * rep;
@@ -658,7 +670,8 @@ int prepare_rq_elems(Rq_coll * clp, const char * inf, const char * outf)
 
 /* Returns a "QS" code and req index, or QS_IDLE and position of first idle
    (-1 if no idle position). Returns -1 on poll error. */
-int decider(Rq_coll * clp, int first_xfer, int * req_indexp)
+static int
+decider(Rq_coll * clp, int first_xfer, int * req_indexp)
 {
     int k, res;
     Rq_elem * rep;
@@ -705,7 +718,8 @@ int decider(Rq_coll * clp, int first_xfer, int * req_indexp)
 }
 
 
-int main(int argc, char * argv[])
+int
+main(int argc, char * argv[])
 {
     int skip = 0;
     int seek = 0;
