@@ -1,30 +1,8 @@
 /*
- * Copyright (c) 2004-2008 Douglas Gilbert.
+ * Copyright (c) 2004-2014 Douglas Gilbert.
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
+ * Use of this source code is governed by a BSD-style
+ * license that can be found in the BSD_LICENSE file.
  */
 
 #include <unistd.h>
@@ -42,14 +20,14 @@
 #include "sg_cmds_basic.h"
 #include "sg_cmds_mmc.h"
 
-/* A utility program for the Linux OS SCSI subsystem.
+/* A utility program originally written for the Linux OS SCSI subsystem.
  *
  * This program outputs information provided by a SCSI "Get Configuration"
    command [0x46] which is only defined for CD/DVDs (in MMC-2,3,4,5,6).
 
 */
 
-static char * version_str = "0.36 20080714";    /* mmc6r02 */
+static const char * version_str = "0.39 20140516";    /* mmc6r02 */
 
 #define MX_ALLOC_LEN 8192
 #define NAME_BUFF_SZ 64
@@ -67,6 +45,7 @@ static struct option long_options[] = {
         {"inner-hex", 0, 0, 'i'},
         {"list", 0, 0, 'l'},
         {"raw", 0, 0, 'R'},
+        {"readonly", 0, 0, 'q'},
         {"rt", 1, 0, 'r'},
         {"starting", 1, 0, 's'},
         {"verbose", 0, 0, 'v'},
@@ -81,9 +60,9 @@ usage()
     fprintf(stderr,
             "Usage:  sg_get_config [--brief] [--current] [--help] [--hex] "
             "[--inner-hex]\n"
-            "                      [--list] [--raw] [--rt=RT] "
-            "[--starting=FC]\n"
-            "                      [--verbose] [--version] DEVICE\n"
+            "                      [--list] [--raw] [--readonly] [--rt=RT]\n"
+            "                      [--starting=FC] [--verbose] [--version] "
+            "DEVICE\n"
             "  where:\n"
             "    --brief|-b       only give feature names of DEVICE "
             "(don't decode)\n"
@@ -96,6 +75,8 @@ usage()
             "    --list|-l        list all known features + profiles "
             "(ignore DEVICE)\n"
             "    --raw|-R         output in binary (to stdout)\n"
+            "    --readonly|-q    open DEVICE read-only (def: open it "
+            "read-write)\n"
             "    --rt=RT|-r RT    default value is 0\n"
             "                     0 -> all feature descriptors (regardless "
             "of currency)\n"
@@ -887,15 +868,15 @@ decode_feature(int feature, unsigned char * ucp, int len)
         num = ucp[5];
         printf("      Profile numbers:\n");
         for (k = 6; (num > 0) && (k < len); --num, k += 2) {
-            printf("        %d\n", 
+            printf("        %d\n",
                    ((unsigned int)ucp[k] << 8) + ucp[k + 1]);
         }
         break;
     default:
-        printf("    Unknown feature [0x%x], version=%d persist=%d, "
-               "current=%d\n", feature, ((ucp[2] >> 2) & 0xf),
-               !!(ucp[2] & 0x2), !!(ucp[2] & 0x1));
-        dStrHex((const char *)ucp, len, 1);
+        fprintf(stderr, "    Unknown feature [0x%x], version=%d persist=%d, "
+                "current=%d\n", feature, ((ucp[2] >> 2) & 0xf),
+                !!(ucp[2] & 0x2), !!(ucp[2] & 0x1));
+        dStrHexErr((const char *)ucp, len, 1);
         break;
     }
 }
@@ -973,6 +954,7 @@ main(int argc, char * argv[])
     int inner_hex = 0;
     int list = 0;
     int do_raw = 0;
+    int readonly = 0;
     int rt = 0;
     int starting = 0;
     int verbose = 0;
@@ -985,7 +967,7 @@ main(int argc, char * argv[])
     while (1) {
         int option_index = 0;
 
-        c = getopt_long(argc, argv, "bchHilr:Rs:vV", long_options,
+        c = getopt_long(argc, argv, "bchHilqr:Rs:vV", long_options,
                         &option_index);
         if (c == -1)
             break;
@@ -1010,6 +992,9 @@ main(int argc, char * argv[])
         case 'l':
             list = 1;
             break;
+        case 'q':
+            ++readonly;
+            break;
         case 'r':
             rt = sg_get_num(optarg);
             if ((rt < 0) || (rt > 3)) {
@@ -1086,7 +1071,7 @@ main(int argc, char * argv[])
     }
     sg_cmds_close_device(sg_fd);
 
-    sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, verbose);
+    sg_fd = sg_cmds_open_device(device_name, readonly, verbose);
     if (sg_fd < 0) {
         fprintf(stderr, ME "open error (rw): %s\n", safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
@@ -1113,16 +1098,11 @@ main(int argc, char * argv[])
         else
             decode_config(resp_buffer, sizeof(resp_buffer), len, brief,
                           inner_hex);
-    } else if (SG_LIB_CAT_INVALID_OP == res)
-        fprintf(stderr, "Get Configuration command not supported\n");
-    else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-        fprintf(stderr, "field in Get Configuration command illegal\n");
-    else if (SG_LIB_CAT_UNIT_ATTENTION == res)
-        fprintf(stderr, "Get Configuration received unit attention\n");
-    else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-        fprintf(stderr, "Get Configuration: aborted command\n");
-    else {
-        fprintf(stderr, "Get Configuration command failed\n");
+    } else {
+        char b[80];
+
+        sg_get_category_sense_str(res, sizeof(b), b, verbose);
+        fprintf(stderr, "Get Configuration command: %s\n", b);
         if (0 == verbose)
             fprintf(stderr, "    try '-v' option for more information\n");
     }
