@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2010 Douglas Gilbert.
+ * Copyright (c) 2005-2014 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -27,7 +27,7 @@
  * DEVICE IDENTIFIER and SET DEVICE IDENTIFIER prior to spc4r07.
  */
 
-static char * version_str = "1.08 20100312";
+static const char * version_str = "1.13 20140904";
 
 #define ME "sg_ident: "
 
@@ -49,10 +49,12 @@ static struct option long_options[] = {
 static void decode_ii(const unsigned char * iip, int ii_len, int itype,
                       int ascii, int raw, int verbose)
 {
-    int k, n;
+    int k;
 
     if (raw) {
         if (ii_len > 0) {
+            int n;
+
             if (sg_set_binary_mode(STDOUT_FILENO) < 0)
                 perror("sg_set_binary_mode");
 #if 0
@@ -60,6 +62,8 @@ static void decode_ii(const unsigned char * iip, int ii_len, int itype,
 #else
             n = write(STDOUT_FILENO, iip, ii_len);
 #endif
+            if (verbose && (n < 1))
+                fprintf(stderr, "unable to write to stdout\n");
         }
         return;
     }
@@ -91,7 +95,8 @@ static void usage()
           "    --clear|-C      clear (set to zero length) identifying "
           "information\n"
           "    --help|-h       print out usage message\n"
-          "    --itype=IT|-i IT    specify information type\n"
+          "    --itype=IT|-i IT    specify identifying information type "
+          "(def: 0)\n"
           "    --raw|-r        output identifying information to "
           "stdout\n"
           "    --set|-S        invoke set identifying information with "
@@ -106,6 +111,7 @@ int main(int argc, char * argv[])
 {
     int sg_fd, res, c, ii_len;
     unsigned char rdi_buff[REPORT_ID_INFO_SANITY_LEN + 4];
+    char b[80];
     unsigned char * ucp = NULL;
     int ascii = 0;
     int do_clear = 0;
@@ -225,27 +231,10 @@ int main(int argc, char * argv[])
             res = sg_ll_set_id_info(sg_fd, itype, rdi_buff, 0, 1, verbose);
         if (res) {
             ret = res;
-            if (SG_LIB_CAT_NOT_READY == res)
-                fprintf(stderr, "Set identifying information command, device "
-                        "not ready\n");
-            else if (SG_LIB_CAT_INVALID_OP == res)
-                fprintf(stderr, "Set identifying information command not "
-                        "supported\n");
-            else if (SG_LIB_CAT_UNIT_ATTENTION == res)
-                fprintf(stderr, "Set identifying information, unit "
-                        "attention\n");
-            else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-                fprintf(stderr, "Set identifying information, aborted "
-                        "command\n");
-            else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-                fprintf(stderr, "bad field in Set identifying information "
-                        "cdb including unsupported service action\n");
-            else {
-                fprintf(stderr, "Set identifying information command "
-                        "failed\n");
-                if (0 == verbose)
-                    fprintf(stderr, "    try '-v' for more information\n");
-            }
+            sg_get_category_sense_str(res, sizeof(b), b, verbose);
+            fprintf(stderr, "Set identifying information: %s\n", b);
+            if (0 == verbose)
+                fprintf(stderr, "    try '-v' for more information\n");
         }
     } else {    /* do report identifying information */
         res = sg_ll_report_id_info(sg_fd, itype, rdi_buff, 4, 1, verbose);
@@ -279,29 +268,10 @@ int main(int argc, char * argv[])
         } else
             ret = res;
         if (ret) {
-            if (SG_LIB_CAT_NOT_READY == ret)
-                fprintf(stderr, "Report identifying information command, "
-                        "device not ready\n");
-            else if (SG_LIB_CAT_UNIT_ATTENTION == ret)
-                fprintf(stderr, "Report identifying information, unit "
-                        "attention\n");
-            else if (SG_LIB_CAT_ABORTED_COMMAND == ret)
-                fprintf(stderr, "Report identifying information, aborted "
-                        "command\n");
-            else if (SG_LIB_CAT_INVALID_OP == ret)
-                fprintf(stderr, "Report identifying information command "
-                        "not supported\n");
-            else if (SG_LIB_CAT_ILLEGAL_REQ == ret)
-                fprintf(stderr, "bad field in Report identifying "
-                        "information cdb including unsupported service "
-                        "action\n");
-            else {
-                fprintf(stderr, "Report identifying information command "
-                        "failed\n");
-                if (0 == verbose)
-                    fprintf(stderr, "    try '-v' for more "
-                            "information\n");
-            }
+            sg_get_category_sense_str(res, sizeof(b), b, verbose);
+            fprintf(stderr, "Report identifying information: %s\n", b);
+            if (0 == verbose)
+                fprintf(stderr, "    try '-v' for more information\n");
         }
     }
 
