@@ -1,30 +1,8 @@
 /*
- * Copyright (c) 2004-2007 Hannes Reinecke, Christophe Varoqui and Douglas Gilbert.
+* Copyright (c) 2004-2013 Hannes Reinecke, Christophe Varoqui, Douglas Gilbert
  * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
+ * Use of this source code is governed by a BSD-style
+ * license that can be found in the BSD_LICENSE file.
  */
 
 #include <unistd.h>
@@ -39,14 +17,14 @@
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
 
-/* A utility program for the Linux OS SCSI subsystem.
+/* A utility program originally written for the Linux OS SCSI subsystem.
  *
  *
  * This program issues the SCSI command SET TARGET PORT GROUPS
  * to the given SCSI device.
  */
 
-static char * version_str = "1.2 20070926";
+static const char * version_str = "1.7 20130730";
 
 #define TGT_GRP_BUFF_LEN 1024
 #define MX_ALLOC_LEN (0xc000 + 0x80)
@@ -58,6 +36,28 @@ static char * version_str = "1.2 20070926";
 #define TPGS_STATE_OFFLINE 0xe          /* SPC-4 rev 9 */
 #define TPGS_STATE_TRANSITIONING 0xf
 
+/* See also table 306 - Target port group descriptor format in SPC-4 rev 36e */
+#ifndef __cplusplus
+
+static const unsigned char state_sup_mask[] = {
+        [TPGS_STATE_OPTIMIZED]     = 0x01,
+        [TPGS_STATE_NONOPTIMIZED]  = 0x02,
+        [TPGS_STATE_STANDBY]       = 0x04,
+        [TPGS_STATE_UNAVAILABLE]   = 0x08,
+        [TPGS_STATE_OFFLINE]       = 0x40,
+        [TPGS_STATE_TRANSITIONING] = 0x80,
+};
+
+#else
+
+// C++ does not support designated initializers
+static const unsigned char state_sup_mask[] = {
+    0x1, 0x2, 0x4, 0x8, 0x0, 0x0, 0x0, 0x0,
+    0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x80,
+};
+
+#endif  /* C or C++ ? */
+
 #define VPD_DEVICE_ID  0x83
 #define DEF_VPD_DEVICE_ID_LEN  252
 
@@ -135,7 +135,7 @@ dStrRaw(const char* str, int len)
 static int
 decode_target_port(unsigned char * buff, int len, int *d_id, int *d_tpg)
 {
-    int c_set, piv, assoc, desig_type, i_len;
+    int c_set, assoc, desig_type, i_len;
     int off, u;
     const unsigned char * ucp;
     const unsigned char * ip;
@@ -153,7 +153,7 @@ decode_target_port(unsigned char * buff, int len, int *d_id, int *d_tpg)
         }
         ip = ucp + 4;
         c_set = (ucp[0] & 0xf);
-        piv = ((ucp[1] & 0x80) ? 1 : 0);
+        /* piv = ((ucp[1] & 0x80) ? 1 : 0); */
         assoc = ((ucp[1] >> 4) & 0x3);
         desig_type = (ucp[1] & 0xf);
         switch (desig_type) {
@@ -161,7 +161,7 @@ decode_target_port(unsigned char * buff, int len, int *d_id, int *d_tpg)
             if ((1 != c_set) || (1 != assoc) || (4 != i_len)) {
                 fprintf(stderr, "      << expected binary code_set, target "
                         "port association, length 4>>\n");
-                dStrHex((const char *)ip, i_len, 0);
+                dStrHexErr((const char *)ip, i_len, 0);
                 break;
             }
             *d_id = ((ip[2] << 8) | ip[3]);
@@ -170,7 +170,7 @@ decode_target_port(unsigned char * buff, int len, int *d_id, int *d_tpg)
             if ((1 != c_set) || (1 != assoc) || (4 != i_len)) {
                 fprintf(stderr, "      << expected binary code_set, target "
                         "port association, length 4>>\n");
-                dStrHex((const char *)ip, i_len, 0);
+                dStrHexErr((const char *)ip, i_len, 0);
                 break;
             }
             *d_tpg = ((ip[2] << 8) | ip[3]);
@@ -229,7 +229,7 @@ transition_tpgs_states(struct tgtgrp *tgtState, int numgrp, int portgroup,
           return 1;
      }
 
-     if (!( (1 << newstate) & tgtState[i].valid )) {
+     if (!( state_sup_mask[newstate] & tgtState[i].valid )) {
           printf("Portgroup 0x%02x: Invalid state 0x%x\n",
                  portgroup, newstate);
           return 1;
@@ -306,7 +306,7 @@ build_port_arr(const char * inp, int * port_arr, int * port_arr_len,
         v = sg_get_num_nomult(lcp);
         if (-1 != v) {
             port_arr[k] = v;
-            cp = strchr(lcp, ',');
+            cp = (char *)strchr(lcp, ',');
             if (NULL == cp)
                 break;
             lcp = cp + 1;
@@ -391,7 +391,7 @@ build_state_arr(const char * inp, int * state_arr, int * state_arr_len,
                 return 1;
             }
         }
-        cp = strchr(lcp, ',');
+        cp = (char *)strchr(lcp, ',');
         if (NULL == cp)
             break;
         lcp = cp + 1;
@@ -408,7 +408,7 @@ build_state_arr(const char * inp, int * state_arr, int * state_arr_len,
 int
 main(int argc, char * argv[])
 {
-    int sg_fd, k, off, res, c, report_len, tgt_port_count, trunc;
+    int sg_fd, k, off, res, c, report_len, tgt_port_count;
     unsigned char reportTgtGrpBuff[TGT_GRP_BUFF_LEN];
     unsigned char setTgtGrpBuff[TGT_GRP_BUFF_LEN];
     unsigned char rsp_buff[MX_ALLOC_LEN + 2];
@@ -423,6 +423,7 @@ main(int argc, char * argv[])
     int port_arr[MAX_PORT_LIST_ARR_LEN];
     int port_arr_len = 0;
     int state_arr[MAX_PORT_LIST_ARR_LEN];
+    char b[80];
     int state_arr_len = 0;
     int portgroup = -1;
     int relport = -1;
@@ -572,29 +573,31 @@ main(int argc, char * argv[])
                         "INQUIRY response\n");
                 if (verbose) {
                     fprintf(stderr, "First 32 bytes of bad response\n");
-                    dStrHex((const char *)rsp_buff, 32, 0);
+                    dStrHexErr((const char *)rsp_buff, 32, 0);
                 }
                 return SG_LIB_CAT_MALFORMED;
             }
             if (report_len > MX_ALLOC_LEN) {
-                fprintf(stderr, "response length too long: %d > %d\n", report_len,
-                        MX_ALLOC_LEN);
+                fprintf(stderr, "response length too long: %d > %d\n",
+                        report_len, MX_ALLOC_LEN);
                 return SG_LIB_CAT_MALFORMED;
             } else if (report_len > DEF_VPD_DEVICE_ID_LEN) {
                 if (sg_ll_inquiry(sg_fd, 0, 1, VPD_DEVICE_ID, rsp_buff,
                                   report_len, 1, verbose))
                     return SG_LIB_CAT_OTHER;
             }
-            decode_target_port(rsp_buff + 4, report_len - 4, &relport, &portgroup);
+            decode_target_port(rsp_buff + 4, report_len - 4, &relport,
+                               &portgroup);
             printf("Device is at port Group 0x%02x, relative port 0x%02x\n",
                    portgroup, relport);
         }
 
         memset(reportTgtGrpBuff, 0x0, sizeof(reportTgtGrpBuff));
-        trunc = 0;
+        /* trunc = 0; */
 
-        res = sg_ll_report_tgt_prt_grp(sg_fd, reportTgtGrpBuff,
-                                sizeof(reportTgtGrpBuff), 1, verbose);
+        res = sg_ll_report_tgt_prt_grp2(sg_fd, reportTgtGrpBuff,
+                                        sizeof(reportTgtGrpBuff), 0, 1,
+                                        verbose);
         ret = res;
         if (0 == res) {
             report_len = (reportTgtGrpBuff[0] << 24) +
@@ -602,7 +605,7 @@ main(int argc, char * argv[])
                          (reportTgtGrpBuff[2] << 8) +
                          reportTgtGrpBuff[3] + 4;
             if (report_len > (int)sizeof(reportTgtGrpBuff)) {
-                trunc = 1;
+                /* trunc = 1; */
                 fprintf(stderr, "  <<report too long for internal buffer,"
                         " output truncated\n");
                 report_len = (int)sizeof(reportTgtGrpBuff);
@@ -615,7 +618,7 @@ main(int argc, char * argv[])
                 printf("Report list length = %d\n", report_len);
             if (hex) {
                 if (verbose)
-                    fprintf(stderr, "\nOutput response in hex:\n");
+                    printf("\nOutput response in hex:\n");
                 dStrHex((const char *)reportTgtGrpBuff, report_len, 1);
                 goto err_out;
             }
@@ -639,17 +642,9 @@ main(int argc, char * argv[])
                 tgtStatePtr++;
                 off = 8 + tgt_port_count * 4;
             }
-        } else if (SG_LIB_CAT_INVALID_OP == res)
-            fprintf(stderr, "Report Target Port Groups command not supported\n");
-        else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-            fprintf(stderr, "bad field in Report Target Port Groups cdb "
-                    "including unsupported service action\n");
-        else if (SG_LIB_CAT_UNIT_ATTENTION == res)
-            fprintf(stderr, "Report Target Port Groups, unit attention\n");
-        else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-            fprintf(stderr, "Report Target Port Groups, aborted command\n");
-        else {
-            fprintf(stderr, "Report Target Port Groups command failed\n");
+        } else {
+            sg_get_category_sense_str(res, sizeof(b), b, verbose);
+            fprintf(stderr, "Report Target Port Groups: %s\n", b);
             if (0 == verbose)
                 fprintf(stderr, "    try '-v' for more information\n");
         }
@@ -663,7 +658,7 @@ main(int argc, char * argv[])
         transition_tpgs_states(tgtGrpState, numgrp, portgroup, state);
 
         memset(setTgtGrpBuff, 0x0, sizeof(setTgtGrpBuff));
-        trunc = 0;
+        /* trunc = 0; */
 
         encode_tpgs_states(setTgtGrpBuff, tgtGrpState, numgrp);
         report_len = numgrp * 4 + 4;
@@ -681,17 +676,9 @@ main(int argc, char * argv[])
 
     if (0 == res)
         goto err_out;
-    else if (SG_LIB_CAT_INVALID_OP == res)
-        fprintf(stderr, "Set Target Port Groups command not supported\n");
-    else if (SG_LIB_CAT_ILLEGAL_REQ == res)
-        fprintf(stderr, "bad field in Report Target Port Groups cdb "
-                "including unsupported service action\n");
-    else if (SG_LIB_CAT_UNIT_ATTENTION == res)
-        fprintf(stderr, "Set Target Port Groups, unit attention\n");
-    else if (SG_LIB_CAT_ABORTED_COMMAND == res)
-        fprintf(stderr, "Set Target Port Groups, aborted command\n");
     else {
-        fprintf(stderr, "Set Target Port Groups command failed\n");
+        sg_get_category_sense_str(res, sizeof(b), b, verbose);
+        fprintf(stderr, "Set Target Port Groups: %s\n", b);
         if (0 == verbose)
             fprintf(stderr, "    try '-v' for more information\n");
     }
